//TCPAgent.java: Copyright (C) 2000 Thomas Repantis <darkzero@otenet.gr>
//
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//

import java.net.*;
import java.io.*;


//TCPAgent listens for client connection requests and hands them in to
//TCPAgentThread to get served. This way concurrency is achieved.

public class TCPAgent {


    public static void main(String[] args) throws IOException {

	//******************************************************************
	//_______________
	//Variables used.
	//---------------

	String host = "localhost";             //Host to connect to, if none supplied.
	Integer timePort = new Integer(37);    //Default server time port.
	Integer chargenPort = new Integer(19); //Default server chargen port.
	Integer port2listen = new Integer(14000); //Default port2listen.
	Integer numOfChars = new Integer(1); //Get only one character from chargen, by default.
	boolean logflag = false;               //Logging off by default. 

	String arg;
	int i = 0;

	ServerSocket serverSocket = null;
        boolean listening = true;

	//******************************************************************

	//******************************************************************
	//___________________
	//Introduce yourself.
	//-------------------

	System.out.println();
	System.out.println("TCPAgent");
	System.out.println("Copyright (C) 2000 Thomas Repantis <darkzero@otenet.gr>");   
	System.out.println("Terminate with Ctrl+C.");
	System.out.println("java TCPAgent -h for help");
	System.out.println();

	//******************************************************************


	//******************************************************************
	//____________________________
	//Do the command line parsing.
	//----------------------------
	
	while (i < args.length && args[i].startsWith("-")) {

            arg = args[i++];
	    
	    if (arg.equals("-host")){
		if (i < args.length)
		    host = args[i++];
		else{
		    System.out.println();
		    System.err.println("-host requires a hostname");
		    System.err.println("java TCPAgent -h for help");
		    System.out.println();
		    System.exit(1);
		}
	    }

	    else if (arg.equals("-timeport")){
		if (i < args.length){
		    try{
			timePort = new Integer(timePort.parseInt(args[i++]));  
		    } catch (NumberFormatException e) {
			System.err.println("Invalid port number.");
			System.exit(1);		
		    }
		}
		else{
		    System.out.println();
		    System.err.println("-timeport requires a portnumber");
		    System.err.println("java TCPAgent -h for help");
		    System.out.println();
		    System.exit(1);
		}
	    }


	    else if (arg.equals("-chargenport")){
		if (i < args.length){
		    try{
			chargenPort = new Integer(chargenPort.parseInt(args[i++])); 
		    } catch (NumberFormatException e) {
			System.err.println("Invalid port number.");
			System.exit(1);		
		    }
		}
		else{
		    System.out.println();
		    System.err.println("-chargenport requires a portnumber");
		    System.err.println("java TCPAgent -h for help");
		    System.out.println();
		    System.exit(1);
		}
	    }
	    

	    else if (arg.equals("-port2listen")){
		if (i < args.length){
		    try{
			port2listen = new Integer(port2listen.parseInt(args[i++]));   
		    } catch (NumberFormatException e) {
			System.err.println("Invalid port number.");
			System.exit(1);		
		    }
		}
		else{
		    System.out.println();
		    System.err.println("-port2listen requires a portnumber");
		    System.err.println("java TCPAgent -h for help");
		    System.out.println();
		    System.exit(1);
		}
	    }


	    else if (arg.equals("-c")){
		if (i < args.length){
		    try{
			numOfChars = new Integer(numOfChars.parseInt(args[i++]));
		    } catch (NumberFormatException e) {
			System.err.println("Invalid number of characters.");
			System.exit(1);
		    }
		}
		else{
		    System.out.println();
		    System.err.println("-c requires a number of characters");
		    System.err.println("java TCPAgent -h for help");
		    System.out.println();
		    System.exit(1);
		}
	    }	 
	    
	    else if (arg.equals("-log")){
		logflag = true;
		System.out.println();	
		System.out.println("Logging enabled. See file TCPAgent.log.");
		System.out.println();
	    }

	    else if (arg.equals("-h")){
		//Give the poor user a little help...
		System.out.println();
		System.out.println("Usage: java TCPAgent [-h] [-c NumberOfChars] [-host hostname] [-chargenport portnumber] [-timeport portnumber] [-port2listen portnumber] [-log]");
		System.out.println();
		System.out.println("TCPAgent will listen for TCPAgentClients' connections on the specified port (or on port 14000 by default) and once triggered will connect to a time and chargen server on the specified host (or on the localhost by default), on the specified ports (or on timeport 37 and on chargenport 19 by default), will get time and chargen output (specified number of characters or one character by default), send them to the TCPAgentClient and log this action in file TCPAgent.log, if invoked with the -log argument.");
		System.out.println();
		System.out.println("This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under certain conditions. TCPAgent is licensed under the GNU General Public License. See the file COPYING for details.");
	    	System.out.println();
		System.exit(1);
	    }
	    
	    else {
		System.err.println("Error in the command line arguments passed.");
		System.err.println("java TCPAgent -h for help");
		System.exit(1);
	    }
	}

	if (i != args.length){
	    System.err.println("Error in the command line arguments passed.");
	    System.err.println("java TCPAgent -h for help");
	    System.exit(1);
	}

	else{
	    System.out.println("Using as a server host: " + host);
	    System.out.println("Using timeport: " + timePort);
	    System.out.println("Using chargenport: " + chargenPort);	    
	    System.out.println("Number of characters to get from chargen: " + numOfChars);	    
	    System.out.println("Listening for connections on port: " + port2listen);
	    if( logflag == true ){
		System.out.println("Logging is enabled.");
	    }
	    if( logflag == false ){
		System.out.println("Logging is disabled.");
	    }
	}
	    
	//******************************************************************
 

	//******************************************************************
	//___________________________________________________
	//Start listening for connections of TCPAgentClients. 
	//---------------------------------------------------


        try {
            serverSocket = new ServerSocket(port2listen.intValue());
	    //The user should normally not need to change the listening port,
	    //since this should be standard for the TCPAgent (an unused
	    //registered port chosen by the implementor (me ;-)). However the
	    //ability to change it is provided, in case something else is
	    //running there.

        } catch (IOException e) {
            System.err.println("Could not listen on port: " + port2listen);
            System.exit(-1);
        }

        while (listening)
            new TCPAgentThread(serverSocket.accept(), host, 
			   timePort, chargenPort, numOfChars, logflag).start();
	//The agent's client just needs to call the agent to get served.
	//The agent should not need any data from the client.
        serverSocket.close();

	//******************************************************************
    }
}




