//TCPAgentThread.java: Copyright (C) 2000 Thomas Repantis <darkzero@otenet.gr>
//
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//

import java.net.*;
import java.io.*;
import java.util.*;

//TCPAgentThread -once called- serves the agent's client. That is, it
//connects to another -standard- server gets TCP time and TCP chargen
//output and sends them back to the agent client.

public class TCPAgentThread extends Thread {


    //******************************************************************
    //_______________
    //Variables used.
    //---------------

    private Socket outputSocket = null;
    private String host = null;
    private Integer timePort = null;
    private Integer chargenPort = null;
    private Integer numOfChars = null;
    private boolean logflag;
    
    private Socket timeSocket = null;
    private Socket chargenSocket = null;

    PrintWriter output = null;
    BufferedInputStream timeInput = null;
    BufferedReader chargenInput = null;

    int time = 0;
    int j = 0;
    StringBuffer time2send = null;        //The whole 32-bit integer output
                                          //of Time is stored here to be
                                          //passed later to the TCPAgentClient.
    StringBuffer chargen2send = null;     //All the characters from chargen
                                          //we have decided with numOfChars
                                          //we want to keep are stored here 
                                          //to be passed later to the
                                          //TCPAgentClient.

    //******************************************************************
    //______________________________________________________________
    //Connect to the time and chargen servers, get the responses and
    //pass them to the TCPAgentClient.
    //--------------------------------------------------------------

    //The same Socket could be used for both
    //-sequentially done- jobs, to be more economical. We use
    //separate just to be on the safe side, in case something goes
    //wrong.

    public TCPAgentThread(Socket outputSocket, String host, Integer 
       timePort, Integer chargenPort, Integer numOfChars, boolean logflag) {
	
	super("TCPAgentThread");
    
	//The Thread is called and given as an argument the socket on which to
	//write the output, which is the agent-clientOfAgent socket.
	this.outputSocket = outputSocket;
	this.host = host;
	this.timePort = timePort;
	this.chargenPort = chargenPort;
	this.numOfChars = numOfChars;
	this.logflag = logflag;
    }

    public void run() {

	try {
	    //The two jobs -to get TCP time and to get TCP chargen-
	    //are done quickly; therefore no concurrency is needed here.
	    

	    //_________________________
	    //Get time from the server.
	    //-------------------------
	    
	    timeSocket = new Socket(host, timePort.intValue());
	    timeInput = new BufferedInputStream(new BufferedInputStream(
					   timeSocket.getInputStream()));
	    StringBuffer time2send = new StringBuffer();

	    while((time = timeInput.read()) != -1){

		//The value byte is returned as an int in the range 0 to 255.
		// If no byte is available because the end of the stream has
		// been reached, the value -1 is returned.

		//Time is integer and the read() method returns integer.
		//Therefore no conversion/casting is required.
		time2send.append(time);

	    }

	   
	    //____________________________
	    //Get chargen from the server.
	    //----------------------------

	    chargenSocket = new Socket(host, chargenPort.intValue());
	    chargenInput = new BufferedReader(new InputStreamReader(
			       		      chargenSocket.getInputStream()));
	    StringBuffer chargen2send = new StringBuffer();

    	    for(j = 0; (j < numOfChars.intValue()); j++) {

		//Chargen is character and the read() method returns integer.
		//Therefore a conversion is required, achieved through casting.
		char c = (char)chargenInput.read();
	    	chargen2send.append(c);

	    }

	    //__________________________________________________________
	    //Pass the time and chargen responses to the TCPAgentClient.
	    //----------------------------------------------------------

	    output = new PrintWriter(outputSocket.getOutputStream(), true);
	    output.println(time2send);
	    output.println(chargen2send);

	    //___________________________________
	    //Log, if you 've been told to do so.
	    //-----------------------------------

	    if( logflag == true ){
                     		
		try{
		    //Successful connections will be logged. Connections
		    //that caused an exception will show up on the System.err.

		    //We preferred to give a standard name to the logfile,
		    //rather than a user defined one, in order to avoid
		    //problems, like overwriting existing irrelevant files
		    // and so on.

		    String logFileName = "TCPAgent.log";
		    File logFile = new File(logFileName);

		    //We use the RandomAccessFile class, so that we can append
		    //data to an existing file.
		    RandomAccessFile log = new RandomAccessFile(logFile, "rw");
		    log.seek(logFile.length());
		  
		    Date logDate = new Date();
		    log.writeChars( logDate.toString() + " -> " + outputSocket.toString());
		    //Write a new line to the logfile.
		    log.writeByte('\n');
		    log.close();
		} catch(IOException e) {

		    System.err.println("Could not write properly to the logfile.\n");
		    System.err.println(e.toString());
		    System.exit(1);

		}	    

	    }

	} catch (UnknownHostException e) {
	    System.err.println("Couldn't locate host.");
	    System.exit(1);
        } catch (IOException e) {
	    System.err.println("Couldn't get I/O for the connection to the host.");
	    System.exit(1);
	} catch (NullPointerException e) {
	    System.err.println("Somebody called the TCPagent without waiting for its response.");
	    System.exit(1);
	}
						

	//Do some housekeeping.
	try {

	timeInput.close();
	chargenInput.close();
       	output.close();

	timeSocket.close();
	chargenSocket.close();
	outputSocket.close();	

	} catch (IOException e) {
	    System.err.println("Couldn't get I/O for the connection to the host.");
            System.exit(1);
	}

	//******************************************************************						
    }
}



